from dataclasses import dataclass, field


@dataclass
class QueryCacheBase:
    """Simple cache that ignores any policy settings"""

    _cache: dict = field(default_factory=dict, init=True)

    def save(self, key: str, data: object, prefix: str = "default", policy: dict = None):
        self._cache[key] = data

    def get(self, key: str, prefix: str = "default", policy: dict = None) -> object:
        return self._cache.get(key, None)

    def clear(
        self,
        key: str,
        prefix: str = "default",
    ):
        del self._cache[key]

    def clear_all(self):
        self._cache = {}

    def cache_size(self) -> int:
        """Returns number of items in cache"""
        return len(self._cache)



from typing import Dict

from .datamodel.cache import QueryCacheBase

default_cache_policy_toplevel = {
    "max_age": 3600,
    "s3": False,
    "fcache": False,
    "memory": False,
}

default_cache_policies_keywords = {
    "bql": {"max_age": 3600, "fcache": True, "memory": True},
    "s3": {"max_age": None, "fcache": True, "memory": False},
    "sklearn": {"max_age": None, "fcache": True, "memory": True},
}


# Add extensions via register_extension()
# Extensions are loaded on first access, to avoid requiring
# unused dependencies

_KNOWN_EXTENSIONS: Dict[str, str] = {
    "bql": "iql.extensions.bql_ext.bql_extension",
    "pandas": "iql.extensions.pandas_extension",
    "cache": "iql.extensions.cache_extension",
    "blpapi": "iql.extensions.blpapi_ext.blp_extension",
}

_LOADED_EXTENSIONS = []

_CACHE = QueryCacheBase()


def get_cache():
    return _CACHE


def set_cache(cache: QueryCacheBase):
    global _CACHE
    _CACHE = cache



import json
import logging
from dataclasses import dataclass
from typing import Optional

import pandas as pd

from ..constants import get_cache
from ..datamodel.extension import IqlExtension
from ..datamodel.subquery import SubQuery
from ..ql import register_extension

logger = logging.getLogger(__name__)


@dataclass
class CacheClearAll(IqlExtension):
    """Clears the entire cache"""

    keyword: str

    def executeimpl(self, sq: SubQuery) -> pd.DataFrame:
        get_cache().clear_all()

        return pd.DataFrame([{"result:": True}])


@dataclass
class CacheClear(IqlExtension):
    """Clears a single item from the cache"""

    keyword: str

    def executeimpl(self, sq: SubQuery) -> pd.DataFrame:
        key: str = sq.options["key"]  # type: ignore

        prefix: str = sq.options.get("prefix", "default")  # type: ignore
        get_cache().clear(key, prefix)

        return pd.DataFrame([{"result:": True}])


@dataclass
class CacheGet(IqlExtension):
    """Gets a single item from the cache"""

    keyword: str

    def executeimpl(self, sq: SubQuery) -> pd.DataFrame:
        key = sq.options["key"]
        prefix = sq.options.get("prefix", "default")
        policy = json.loads(sq.options["policy"]) if "policy" in sq.options else None  # type: ignore

        cached_df: pd.DataFrame = get_cache().get(key, prefix, policy)  # type: ignore
        return cached_df


@dataclass
class CachePut(IqlExtension):
    """Clears a single item from the cache"""

    keyword: str

    def executeimpl(self, sq: SubQuery) -> pd.DataFrame:
        prefix = "default"
        data: Optional[pd.DataFrame] = None

        query = sq.options["query"]
        key = sq.options["key"]

        if "prefix" in sq.options:
            prefix = sq.options["prefix"]

        data = sq.iqc.db.execute_query(query, context=sq.iqc.context)  # type: ignore

        if "policy" in sq.options:
            policy = json.loads(sq.options["policy"])  # type: ignore
        else:
            policy = None

        get_cache().save(key, data, prefix, policy)  # type: ignore
        return pd.DataFrame([{"result:": True}])


def register(keyword):
    register_extension(CacheClearAll(keyword=keyword, subword="clear_all"))
    register_extension(CacheClear(keyword=keyword, subword="clear"))
    register_extension(CacheGet(keyword=keyword, subword="get"))
    register_extension(CachePut(keyword=keyword, subword="put"))



def load_ipython_extension(ip):
    """Defer imports for non-ipython environments"""
    from IPython.core.getipython import get_ipython

    from .iql_magic import IqlMagic

    ip = get_ipython()
    ip.register_magics(IqlMagic)  # type: ignore
